@@ -1,4 +1,5 @@
 """Wildcard parsing."""
+from __future__ import annotations
 import re
 import functools
 import bracex
@@ -6,7 +7,7 @@ import os
 from . import util
 from . import posix
 from . _wcmatch import WcRegexp
-from typing import List, Tuple, AnyStr, Iterable, Pattern, Generic, Optional, Set, Sequence, Union, overload, cast
+from typing import AnyStr, Iterable, Pattern, Generic, Sequence, overload, cast
 
 UNICODE_RANGE = '\u0000-\U0010ffff'
 ASCII_RANGE = '\x00-\xff'
@@ -281,16 +282,16 @@ class PatternLimitException(Exception):
 
 
 @overload
-def iter_patterns(patterns: Union[str, Sequence[str]]) -> Iterable[str]:
+def iter_patterns(patterns: str | Sequence[str]) -> Iterable[str]:
     ...
 
 
 @overload
-def iter_patterns(patterns: Union[bytes, Sequence[bytes]]) -> Iterable[bytes]:
+def iter_patterns(patterns: bytes | Sequence[bytes]) -> Iterable[bytes]:
     ...
 
 
-def iter_patterns(patterns: Union[AnyStr, Sequence[AnyStr]]) -> Iterable[AnyStr]:
+def iter_patterns(patterns: AnyStr | Sequence[AnyStr]) -> Iterable[AnyStr]:
     """Return a simple string sequence."""
 
     if isinstance(patterns, (str, bytes)):
@@ -299,7 +300,7 @@ def iter_patterns(patterns: Union[AnyStr, Sequence[AnyStr]]) -> Iterable[AnyStr]
         yield from patterns
 
 
-def escape(pattern: AnyStr, unix: Optional[bool] = None, pathname: bool = True, raw: bool = False) -> AnyStr:
+def escape(pattern: AnyStr, unix: bool | None = None, pathname: bool = True, raw: bool = False) -> AnyStr:
     """
     Escape.
 
@@ -350,7 +351,7 @@ def _get_win_drive(
     pattern: str,
     regex: bool = False,
     case_sensitive: bool = False
-) -> Tuple[bool, Optional[str], bool, int]:
+) -> tuple[bool, str | None, bool, int]:
     """Get Windows drive."""
 
     drive = None
@@ -374,10 +375,10 @@ def _get_win_drive(
             complete = 1
             first = 1
             count = 0
-            for count, m in enumerate(RE_WIN_DRIVE_PART.finditer(pattern, m.end(0)), 1):
-                end = m.end(0)
-                part.append(RE_WIN_DRIVE_UNESCAPE.sub(r'\1', m.group(1)))
-                slash = bool(m.group(2))
+            for count, m2 in enumerate(RE_WIN_DRIVE_PART.finditer(pattern, m.end(0)), 1):
+                end = m2.end(0)
+                part.append(RE_WIN_DRIVE_UNESCAPE.sub(r'\1', m2.group(1)))
+                slash = bool(m2.group(2))
                 if is_special:
                     if count == first and part[-1].lower() == 'unc':
                         complete += 2
@@ -397,7 +398,7 @@ def _get_win_drive(
     return root_specified, drive, slash, end
 
 
-def _get_magic_symbols(pattern: AnyStr, unix: bool, flags: int) -> Tuple[Set[AnyStr], Set[AnyStr]]:
+def _get_magic_symbols(pattern: AnyStr, unix: bool, flags: int) -> tuple[set[AnyStr], set[AnyStr]]:
     """Get magic symbols."""
 
     if isinstance(pattern, bytes):
@@ -407,28 +408,28 @@ def _get_magic_symbols(pattern: AnyStr, unix: bool, flags: int) -> Tuple[Set[Any
         ptype = util.UNICODE
         slash = '\\'
 
-    magic = set()  # type: Set[AnyStr]
+    magic = set()  # type: set[AnyStr]
     if unix:
-        magic_drive = set()  # type: Set[AnyStr]
+        magic_drive = set()  # type: set[AnyStr]
     else:
-        magic_drive = set([slash])
+        magic_drive = {slash}
 
-    magic |= cast(Set[AnyStr], MAGIC_DEF[ptype])
+    magic |= cast('set[AnyStr]', MAGIC_DEF[ptype])
     if flags & BRACE:
-        magic |= cast(Set[AnyStr], MAGIC_BRACE[ptype])
-        magic_drive |= cast(Set[AnyStr], MAGIC_BRACE[ptype])
+        magic |= cast('set[AnyStr]', MAGIC_BRACE[ptype])
+        magic_drive |= cast('set[AnyStr]', MAGIC_BRACE[ptype])
     if flags & SPLIT:
-        magic |= cast(Set[AnyStr], MAGIC_SPLIT[ptype])
-        magic_drive |= cast(Set[AnyStr], MAGIC_SPLIT[ptype])
+        magic |= cast('set[AnyStr]', MAGIC_SPLIT[ptype])
+        magic_drive |= cast('set[AnyStr]', MAGIC_SPLIT[ptype])
     if flags & GLOBTILDE:
-        magic |= cast(Set[AnyStr], MAGIC_TILDE[ptype])
+        magic |= cast('set[AnyStr]', MAGIC_TILDE[ptype])
     if flags & EXTMATCH:
-        magic |= cast(Set[AnyStr], MAGIC_EXTMATCH[ptype])
+        magic |= cast('set[AnyStr]', MAGIC_EXTMATCH[ptype])
     if flags & NEGATE:
         if flags & MINUSNEGATE:
-            magic |= cast(Set[AnyStr], MAGIC_MINUS_NEGATE[ptype])
+            magic |= cast('set[AnyStr]', MAGIC_MINUS_NEGATE[ptype])
         else:
-            magic |= cast(Set[AnyStr], MAGIC_NEGATE[ptype])
+            magic |= cast('set[AnyStr]', MAGIC_NEGATE[ptype])
 
     return magic, magic_drive
 
@@ -504,7 +505,7 @@ def expand_braces(patterns: AnyStr, flags: int, limit: int) -> Iterable[AnyStr]:
             try:
                 # Turn off limit as we are handling it ourselves.
                 yield from bracex.iexpand(p, keep_escapes=True, limit=limit)
-            except bracex.ExpansionLimitException:
+            except bracex.ExpansionLimitException:  # noqa: PERF203
                 raise
             except Exception:  # pragma: no cover
                 # We will probably never hit this as `bracex`
@@ -595,34 +596,34 @@ def no_negate_flags(flags: int) -> int:
 
 @overload
 def translate(
-    patterns: Union[str, Sequence[str]],
+    patterns: str | Sequence[str],
     flags: int,
     limit: int = PATTERN_LIMIT,
-    exclude: Optional[Union[str, Sequence[str]]] = None
-) -> Tuple[List[str], List[str]]:
+    exclude: str | Sequence[str] | None = None
+) -> tuple[list[str], list[str]]:
     ...
 
 
 @overload
 def translate(
-    patterns: Union[bytes, Sequence[bytes]],
+    patterns: bytes | Sequence[bytes],
     flags: int,
     limit: int = PATTERN_LIMIT,
-    exclude: Optional[Union[bytes, Sequence[bytes]]] = None
-) -> Tuple[List[bytes], List[bytes]]:
+    exclude: bytes | Sequence[bytes] | None = None
+) -> tuple[list[bytes], list[bytes]]:
     ...
 
 
 def translate(
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     flags: int,
     limit: int = PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
-) -> Tuple[List[AnyStr], List[AnyStr]]:
+    exclude: AnyStr | Sequence[AnyStr] | None = None
+) -> tuple[list[AnyStr], list[AnyStr]]:
     """Translate patterns."""
 
-    positive = []  # type: List[AnyStr]
-    negative = []  # type: List[AnyStr]
+    positive = []  # type: list[AnyStr]
+    negative = []  # type: list[AnyStr]
 
     if exclude is not None:
         flags = no_negate_flags(flags)
@@ -639,7 +640,8 @@ def translate(
         for pattern in iter_patterns(patterns):
             pattern = util.norm_pattern(pattern, not is_unix, bool(flags & RAWCHARS))
             count = 0
-            for count, expanded in enumerate(expand(pattern, flags, current_limit), 1):
+            for expanded in expand(pattern, flags, current_limit):
+                count += 1
                 total += 1
                 if 0 < limit < total:
                     raise PatternLimitException("Pattern limit exceeded the limit of {:d}".format(limit))
@@ -653,8 +655,8 @@ def translate(
                 current_limit -= count
                 if current_limit < 1:
                     current_limit = 1
-    except bracex.ExpansionLimitException:
-        raise PatternLimitException("Pattern limit exceeded the limit of {:d}".format(limit))
+    except bracex.ExpansionLimitException as e:
+        raise PatternLimitException("Pattern limit exceeded the limit of {:d}".format(limit)) from e
 
     if negative and not positive:
         if flags & NEGATEALL:
@@ -681,34 +683,34 @@ def split(pattern: AnyStr, flags: int) -> Iterable[AnyStr]:
 
 @overload
 def compile_pattern(
-    patterns: Union[str, Sequence[str]],
+    patterns: str | Sequence[str],
     flags: int,
     limit: int = PATTERN_LIMIT,
-    exclude: Optional[Union[str, Sequence[str]]] = None
-) -> Tuple[List[Pattern[str]], List[Pattern[str]]]:
+    exclude: str | Sequence[str] | None = None
+) -> tuple[list[Pattern[str]], list[Pattern[str]]]:
     ...
 
 
 @overload
 def compile_pattern(
-    patterns: Union[bytes, Sequence[bytes]],
+    patterns: bytes | Sequence[bytes],
     flags: int,
     limit: int = PATTERN_LIMIT,
-    exclude: Optional[Union[bytes, Sequence[bytes]]] = None
-) -> Tuple[List[Pattern[bytes]], List[Pattern[bytes]]]:
+    exclude: bytes | Sequence[bytes] | None = None
+) -> tuple[list[Pattern[bytes]], list[Pattern[bytes]]]:
     ...
 
 
 def compile_pattern(
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     flags: int,
     limit: int = PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
-) -> Tuple[List[Pattern[AnyStr]], List[Pattern[AnyStr]]]:
+    exclude: AnyStr | Sequence[AnyStr] | None = None
+) -> tuple[list[Pattern[AnyStr]], list[Pattern[AnyStr]]]:
     """Compile the patterns."""
 
-    positive = []  # type: List[Pattern[AnyStr]]
-    negative = []  # type: List[Pattern[AnyStr]]
+    positive = []  # type: list[Pattern[AnyStr]]
+    negative = []  # type: list[Pattern[AnyStr]]
 
     if exclude is not None:
         flags = no_negate_flags(flags)
@@ -724,7 +726,8 @@ def compile_pattern(
         for pattern in iter_patterns(patterns):
             pattern = util.norm_pattern(pattern, not is_unix, bool(flags & RAWCHARS))
             count = 0
-            for count, expanded in enumerate(expand(pattern, flags, current_limit), 1):
+            for expanded in expand(pattern, flags, current_limit):
+                count += 1
                 total += 1
                 if 0 < limit < total:
                     raise PatternLimitException("Pattern limit exceeded the limit of {:d}".format(limit))
@@ -738,8 +741,8 @@ def compile_pattern(
                 current_limit -= count
                 if current_limit < 1:
                     current_limit = 1
-    except bracex.ExpansionLimitException:
-        raise PatternLimitException("Pattern limit exceeded the limit of {:d}".format(limit))
+    except bracex.ExpansionLimitException as e:
+        raise PatternLimitException("Pattern limit exceeded the limit of {:d}".format(limit)) from e
 
     if negative and not positive:
         if flags & NEGATEALL:
@@ -755,29 +758,29 @@ def compile_pattern(
 
 @overload
 def compile(  # noqa: A001
-    patterns: Union[str, Sequence[str]],
+    patterns: str | Sequence[str],
     flags: int,
     limit: int = PATTERN_LIMIT,
-    exclude: Optional[Union[str, Sequence[str]]] = None
+    exclude: str | Sequence[str] | None = None
 ) -> WcRegexp[str]:
     ...
 
 
 @overload
 def compile(  # noqa: A001
-    patterns: Union[bytes, Sequence[bytes]],
+    patterns: bytes | Sequence[bytes],
     flags: int,
     limit: int = PATTERN_LIMIT,
-    exclude: Optional[Union[bytes, Sequence[bytes]]] = None
+    exclude: bytes | Sequence[bytes] | None = None
 ) -> WcRegexp[bytes]:
     ...
 
 
 def compile(  # noqa: A001
-    patterns: Union[AnyStr, Sequence[AnyStr]],
+    patterns: AnyStr | Sequence[AnyStr],
     flags: int,
     limit: int = PATTERN_LIMIT,
-    exclude: Optional[Union[AnyStr, Sequence[AnyStr]]] = None
+    exclude: AnyStr | Sequence[AnyStr] | None = None
 ) -> WcRegexp[AnyStr]:
     """Compile patterns."""
 
@@ -816,17 +819,17 @@ class WcSplit(Generic[AnyStr]):
         if c in ('^', '-', '['):
             c = next(i)
 
-        while c != ']':
-            if c == '\\':
-                # Handle escapes
-                try:
+        try:
+            while c != ']':
+                if c == '\\':
+                    # Handle escapes
                     self._references(i, True)
-                except PathNameException:
-                    raise StopIteration
-            elif c == '/':
-                if self.pathname:
-                    raise StopIteration
-            c = next(i)
+                elif c == '/':
+                    if self.pathname:
+                        raise StopIteration
+                c = next(i)
+        except PathNameException as e:
+            raise StopIteration from e
 
     def _references(self, i: util.StringIter, sequence: bool = False) -> None:
         """Handle references."""
@@ -954,7 +957,7 @@ class WcParse(Generic[AnyStr]):
         self.unix = is_unix_style(self.flags)
         if not self.unix:
             self.win_drive_detect = self.pathname
-            self.char_avoid = (ord('\\'), ord('/'), ord('.'))  # type: Tuple[int, ...]
+            self.char_avoid = (ord('\\'), ord('/'), ord('.'))  # type: tuple[int, ...]
             self.bslash_abort = self.pathname
             sep = {"sep": re.escape('\\/')}
         else:
@@ -1028,7 +1031,7 @@ class WcParse(Generic[AnyStr]):
 
         return value
 
-    def _sequence_range_check(self, result: List[str], last: str) -> bool:
+    def _sequence_range_check(self, result: list[str], last: str) -> bool:
         """
         If range backwards, remove it.
 
@@ -1053,7 +1056,7 @@ class WcParse(Generic[AnyStr]):
             result.append(last)
         return removed
 
-    def _handle_posix(self, i: util.StringIter, result: List[str], end_range: int) -> bool:
+    def _handle_posix(self, i: util.StringIter, result: list[str], end_range: int) -> bool:
         """Handle posix classes."""
 
         last_posix = False
@@ -1127,8 +1130,8 @@ class WcParse(Generic[AnyStr]):
                     value = self._references(i, True)
                 except DotException:
                     value = re.escape(next(i))
-                except PathNameException:
-                    raise StopIteration
+                except PathNameException as e:
+                    raise StopIteration from e
             elif c == '/':
                 if self.pathname:
                     raise StopIteration
@@ -1210,13 +1213,14 @@ class WcParse(Generic[AnyStr]):
 
         return value
 
-    def _handle_dot(self, i: util.StringIter, current: List[str]) -> None:
+    def _handle_dot(self, i: util.StringIter, current: list[str]) -> None:
         """Handle dot."""
 
         is_current = True
         is_previous = False
 
         if self.after_start and self.pathname and self.nodotdir:
+            index = 0
             try:
                 index = i.index
                 while True:
@@ -1236,16 +1240,16 @@ class WcParse(Generic[AnyStr]):
                             is_current = False
                             is_previous = False
                             raise StopIteration
-                        except DotException:
+                        except DotException as e:
                             if is_current:
                                 is_previous = True
                                 is_current = False
                                 c = next(i)
                             else:
                                 is_previous = False
-                                raise StopIteration
-                        except PathNameException:
-                            raise StopIteration
+                                raise StopIteration from e
+                        except PathNameException as e:
+                            raise StopIteration from e
                     elif c == '/':
                         raise StopIteration
                     else:
@@ -1260,7 +1264,7 @@ class WcParse(Generic[AnyStr]):
         else:
             current.append(re.escape('.'))
 
-    def _handle_star(self, i: util.StringIter, current: List[str]) -> None:
+    def _handle_star(self, i: util.StringIter, current: list[str]) -> None:
         """Handle star."""
 
         if self.pathname:
@@ -1354,7 +1358,7 @@ class WcParse(Generic[AnyStr]):
         else:
             current.append(value)
 
-    def clean_up_inverse(self, current: List[str], nested: bool = False) -> None:
+    def clean_up_inverse(self, current: list[str], nested: bool = False) -> None:
         """
         Clean up current.
 
@@ -1385,7 +1389,7 @@ class WcParse(Generic[AnyStr]):
             index -= 1
         self.inv_ext = 0
 
-    def parse_extend(self, c: str, i: util.StringIter, current: List[str], reset_dot: bool = False) -> bool:
+    def parse_extend(self, c: str, i: util.StringIter, current: list[str], reset_dot: bool = False) -> bool:
         """Parse extended pattern lists."""
 
         # Save state
@@ -1404,7 +1408,7 @@ class WcParse(Generic[AnyStr]):
         success = True
         index = i.index
         list_type = c
-        extended = []  # type: List[str]
+        extended = []  # type: list[str]
 
         try:
             c = next(i)
@@ -1431,7 +1435,8 @@ class WcParse(Generic[AnyStr]):
                         extended.append(self._restrict_extended_slash())
                     extended.append(self.sep)
                 elif c == "|":
-                    self.clean_up_inverse(extended, temp_inv_nest and self.inv_nest)
+                    if self.inv_nest:
+                        self.clean_up_inverse(extended, temp_inv_nest)
                     extended.append(c)
                     if temp_after_start:
                         self.set_start_dir()
@@ -1534,7 +1539,7 @@ class WcParse(Generic[AnyStr]):
         except StopIteration:
             pass
 
-    def root(self, pattern: str, current: List[str]) -> None:
+    def root(self, pattern: str, current: list[str]) -> None:
         """Start parsing the pattern."""
 
         self.set_after_start()
@@ -1551,7 +1556,7 @@ class WcParse(Generic[AnyStr]):
                 self.consume_path_sep(i)
             elif drive is None and root_specified:
                 root_specified = True
-        elif not self.win_drive_detect and self.pathname and pattern.startswith('/'):
+        elif self.pathname and pattern.startswith('/'):
             root_specified = True
 
         if self.no_abs and root_specified:
