@@ -1,5 +1,5 @@
 """Compatibility module."""
-from __future__ import unicode_literals
+from __future__ import annotations
 import sys
 import os
 import stat
@@ -7,9 +7,11 @@ import re
 import unicodedata
 from functools import wraps
 import warnings
-from typing import Any, Callable, Tuple, AnyStr, Match, Pattern, Optional, cast
+from typing import Any, Callable, AnyStr, Match, Pattern, cast
 
 PY310 = (3, 10) <= sys.version_info
+PY312 = (3, 12) <= sys.version_info
+PY313 = (3, 13) <= sys.version_info
 
 UNICODE = 0
 BYTES = 1
@@ -76,7 +78,7 @@ def is_case_sensitive() -> bool:
     return CASE_FS
 
 
-def norm_pattern(pattern: AnyStr, normalize: Optional[bool], is_raw_chars: bool, ignore_escape: bool = False) -> AnyStr:
+def norm_pattern(pattern: AnyStr, normalize: bool | None, is_raw_chars: bool, ignore_escape: bool = False) -> AnyStr:
     r"""
     Normalize pattern.
 
@@ -147,7 +149,7 @@ class StringIter:
 
         return self.iternext()
 
-    def match(self, pattern: Pattern[str]) -> Optional[Match[str]]:
+    def match(self, pattern: Pattern[str]) -> Match[str] | None:
         """Perform regex match at index."""
 
         m = pattern.match(self._string, self._index)
@@ -185,8 +187,8 @@ class StringIter:
         try:
             char = self._string[self._index]
             self._index += 1
-        except IndexError:  # pragma: no cover
-            raise StopIteration
+        except IndexError as e:  # pragma: no cover
+            raise StopIteration from e
 
         return char
 
@@ -194,7 +196,7 @@ class StringIter:
 class Immutable:
     """Immutable."""
 
-    __slots__: Tuple[Any, ...] = tuple()
+    __slots__: tuple[Any, ...] = ()
 
     def __init__(self, **kwargs: Any) -> None:
         """Initialize."""
